
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

const ID_STATUS_BAR = "status-bar";
const ID_POPUPSET = "prefswitch-popupset";
const ID_MENUPOPUP = "prefswitch-menupopup";
const ID_PROFILE_MENUPOPUP = "prefswitch-profile-menupopup";
const ID_PROFILE_SEPARATOR = "prefswitch-profile-separator";
const ID_SHOWHIDE_MENUPOPUP = "prefswitch-showHide-menupopup";
const ID_CSS_POPUP = "prefswitch-css-popup";
const ID_JS_POPUP = "prefswitch-js-popup";
const ID_SEPARATOR = "prefswitch-separator";
const CLASS_INITIAL = "prefswitch-initial";
const CLASS_INJECT = "prefswitch-inject";

const INJECT_PATH = PREF_PATH + "/inject";

function Overlay()
{
	this.appcontent = null;
	this.control = null;
	this.statusbar = null;
	this.popupset = null;
	this.profileMenupopup = null;
	this.profileSeparator = null;
	this.cssPopup = null;
	this.jsPopup = null;
	this.separator = null;
	this.observer = null;
	this.jsObject = {};

	this.openDialog = function(dialogName)
	{
		var url = null;
		var name = null;
		var features = null;
		switch(dialogName)
		{
			case "profile":
				url = "chrome://prefswitch/content/profiledialog.xul";
				name = "profiledialog";
				features = "modal";
				break;

			case "switch":
				url = "chrome://prefswitch/content/switchdialog.xul";
				name = "switchdialog";
				features = "";
				break;

			default: return;
		}
		window.openDialog(url, name, features, this.control);
	};

	this.handleEvent = function(event)
	{
		switch(event.type)
		{
			case "load":
				this.control = new Control(this);
				this.appcontent = $("appcontent");
				this.appcontent.addEventListener("DOMContentLoaded", this, false);
				this.statusbar = $(ID_STATUS_BAR);
				this.popupset = $(ID_POPUPSET);
				this.profileMenupopup = $(ID_PROFILE_MENUPOPUP);
				this.profileSeparator = $(ID_PROFILE_SEPARATOR);
				this.showHideMenupopup = $(ID_SHOWHIDE_MENUPOPUP);
				this.cssPopup = $(ID_CSS_POPUP);
				this.jsPopup = $(ID_JS_POPUP);
				this.separator = $(ID_SEPARATOR);
				this.updateAll();
				this.observer = new Observer(this.control, UPDATE_ALL, ROOT + REFRESH + ".", VALUE);
				this.observer.register();
				break;

/*			case "unload":
				this.appcontent.removeEventListener("DOMContentLoaded", this.updateContentLoaded, false);
				var statusbarpanel = this.statusbar.firstChild;
				while (statusbarpanel)
				{
					if (statusbarpanel.getAttribute("class") == CLASS_INITIAL) statusbarpanel.o.unregister();
					statusbarpanel = statusbarpanel.nextSibling;
				}
				break;*/

			case "DOMContentLoaded":
				this.injectJavaScript(event.target.defaultView);
		}
	};

	this.updateAll = function()
	{
		var profile = this.updateProfiles();

		this.removePopups();
		this.removeStatusbarpanels();
		this.removeMenuitems(this.cssPopup);
		this.removeMenuitems(this.jsPopup);

		this.addStatusbarpanels();
		this.addMenuitems();

		this.loadProfile(profile);
	};

	this.cleanup = function()
	{
		this.observer.disable();

		// BACKUP
		var profileID = this.control.getValue(PROFILE_ID, null, STRING);
		var prefObj = {};
		var popups = [this.cssPopup.firstChild, this.jsPopup.firstChild];
		for (var i = 0; i < popups.length; i++)
		{
			var m = popups[i];
			while (m)
			{
				var key = m.sw.getID() + "." + VALUE;
				prefObj[key] = this.control.getValue(ROOT + key, null, STRING);
				m = m.nextSibling;
			}
		}

		// CLEANUP
		this.control.prefBranch.deleteBranch(ROOT);

		// RESTORE
		this.control.setValue(PROFILE_ID, profileID, STRING);
		for (var key in prefObj) this.control.setValue(ROOT + key, prefObj[key], STRING);

		this.updateAll();
		this.observer.enable();
	};

	this.removeStatusbarpanels = function()
	{
		var element = this.separator.previousSibling;
		while (element.getAttribute("class") == CLASS_INITIAL)
		{
			var next = element.previousSibling;
			element.observer.unregister();
			this.statusbar.removeChild(element);
			element = next;
		}
	};

	this.removePopups = function()
	{
		var popup = this.popupset.firstChild;
		while (popup)
		{
			var next = popup.nextSibling;
			if (popup.tagName == "popup") this.popupset.removeChild(popup);
			popup = next;
		}
	};

	this.removeMenuitems = function(element)
	{
		var menuitem = element.firstChild;
		while (menuitem)
		{
			var next = menuitem.nextSibling;
			menuitem.observer.unregister();
			element.removeChild(menuitem);
			menuitem = next;
		}
	};

	this.addStatusbarpanels = function()
	{
		// REMOVE SHOW-HIDE-MENUITEMS
		while (this.showHideMenupopup.firstChild) this.showHideMenupopup.removeChild(this.showHideMenupopup.firstChild);
		var aSwitch = this.control.getSwitches();

		// CREATE STATUSBARPANELS
		while (aSwitch.length > 0)
		{
			var sw = aSwitch.shift();
			if (sw.isHidden()) continue;
			var statusbarpanel = this.createSwitchElement(sw, "statusbarpanel", CLASS_INITIAL, sw.getInitial(), "onclick");
			statusbarpanel.setAttribute("hidden", "true");
			statusbarpanel.setAttribute("context", ID_MENUPOPUP);
			this.statusbar.insertBefore(statusbarpanel, this.separator);

			// CREATE SHOW-HIDE-MENUITEMS
			var menuitem = document.createElement("menuitem");
			menuitem.setAttribute("type", "checkbox");
			menuitem.setAttribute("label", sw.getName());
//			menuitem.setAttribute("checked", !statusbarpanel.getAttribute("hidden"));
			menuitem.setAttribute("checked", "false");
			statusbarpanel.mi = menuitem;
			menuitem.sp = statusbarpanel;
			menuitem.setAttribute("oncommand", "this.sp.setAttribute('hidden', !this.getAttribute('checked'));");
			this.showHideMenupopup.appendChild(menuitem);
		}
	};

	this.addMenuitems = function()
	{
		// GET FILES
		var aFiles = [];
		var directory = this.control.getDirectory(INJECT_PATH);
		if (directory)
		{
			var entries = directory.directoryEntries;
			while (entries.hasMoreElements())
			{
				var entry = entries.getNext();
				entry.QueryInterface(Components.interfaces.nsIFile);
				if (!entry.isFile() || !entry.leafName.match(/\.(css|js)$/i)) continue;
				aFiles.push([entry.leafName.replace(/\./g, "_"), entry.leafName.replace(/^\d+/, "").replace(/\.\w+$/, ""), entry.path]);
			}
		}

		// SORT FILES
		aFiles.sort();

		// CREATE MENUITEMS
		while (aFiles.length > 0)
		{
			var file = aFiles.shift();
			var sw = new Switch(file[0], file[1], "-");
			sw.setInject(true);
			sw.setInjectPath(file[2]);
			var disabledState = new State("extract-" + file[1], "extract-" + file[1]);
			disabledState.addPreference(new Preference(ROOT + file[0] + "." + VALUE, BOOLEAN, false));
			sw.addState(disabledState);
			var enabledState = new State("inject-" + file[1], "inject-" + file[1]);
			enabledState.addPreference(new Preference(ROOT + file[0] + "." + VALUE, BOOLEAN, true));
			sw.addState(enabledState);

			var menuitem = this.createSwitchElement(sw, "menuitem", null, sw.getName(), "oncommand");
			menuitem.setAttribute("type", "checkbox");
			switch(sw.getInjectType())
			{
				case CSS:
					this.cssPopup.appendChild(menuitem);
					break;

				case JS:
					this.jsPopup.appendChild(menuitem);
			}
		}
	};

	this.createSwitchElement = function(sw, elementType, className, labelName, eventName)
	{
		var element = document.createElement(elementType);
		element.sw = sw;
		element.observer = new Observer(this.control, UPDATE_ONE, ROOT + sw.getID() + ".", VALUE, element);
		element.observer.register();
		if (className) element.setAttribute("class", className);
		element.setAttribute("label", labelName);

		// APPEND CLICK EVENT OR CREATE POPUP MENU
		if (sw.isBooleanSwitch()) element.setAttribute(eventName, "prefswitchOverlay.onclick(event, this.sw, null, true)");
		else
		{
			var popup = document.createElement("popup");
			var id = "prefswitch-popup-" + sw.getID();
			popup.setAttribute("id", id);

			// ADD MENUITEMS
			for (var i = 0; i < sw.getStates().length; i++)
			{
				var st = sw.getState(i);
				var menuitem = document.createElement("menuitem");
				menuitem.setAttribute("type", "radio");
				menuitem.setAttribute("name", "prefswitch-radio-" + sw.getID());
//				menuitem.setAttribute("label", st.getDescription());
				menuitem.setAttribute("label", st.getName());
				menuitem.sw = sw;
				menuitem.st = st;
				st.menuitem = menuitem;
				menuitem.setAttribute("oncommand", "prefswitchOverlay.onclick(event, this.sw, this.st, false)");
				popup.appendChild(menuitem);
			}
			this.popupset.appendChild(popup);
			element.setAttribute("popup", id);
		}
		this.updateOne(element);
		return element;
	};

	this.updateOne = function(element)
	{
		if (!element) return;
		var sw = element.sw;
		if (!sw) return;
		var value = this.control.getValue(ROOT + sw.getID() + "." + VALUE, sw.getState(0).getID(), STRING);
		var st = sw.getStateByID(value);
		if (!st) return;
		switch(element.tagName)
		{
			case "statusbarpanel":
				element.setAttribute("style", "color:" + st.getColor());
				var tooltiptext = st.getTooltip();
				if (tooltiptext != "") element.setAttribute("tooltiptext", tooltiptext);
				if (st.menuitem)
				{
					var menuitem = st.menuitem.parentNode.firstChild;
					while (menuitem)
					{
						if (st.menuitem == menuitem) menuitem.setAttribute("checked", "true");
						else menuitem.setAttribute("checked", "false");
						menuitem = menuitem.nextSibling;
					}
				}
				break;

			case "menuitem":
				var path = sw.getInjectPath();
				var inject = st.getPreference(0).getValue() == "true";
				switch(sw.getInjectType())
				{
					case CSS:
						this.injectCSS(path, inject);
						break;

					case JS: this.setJavascript(sw.getID(), path, inject);
						break;

					default: return;
				}
				element.setAttribute("checked", "" + inject);
		}
	};

	this.injectCSS = function(path, value)
	{
		var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var uri = ios.newURI("file://" + path, CHARSET, null); // URI, CHARSET, BASEURI
		var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"].getService(Components.interfaces.nsIStyleSheetService);
		if (value) { if (!sss.sheetRegistered(uri, sss.USER_SHEET)) sss.loadAndRegisterSheet(uri, sss.USER_SHEET); }
		else { if (sss.sheetRegistered(uri, sss.USER_SHEET)) sss.unregisterSheet(uri, sss.USER_SHEET); }
	};

	this.setJavascript = function(key, path, value)
	{
		if (value) this.jsObject[key] = path;
		else this.jsObject[key] = null;
	};

	this.injectJavaScript = function(defaultView)
	{
		if (!defaultView || defaultView.location.protocol.match(/^about:$/i)) return;
		for (var key in this.jsObject)
		{
			var path = this.jsObject[key];
			if (!path) continue;
			var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
			file.initWithPath(path);
			var lineFinder = null;
			try{
				var data = this.control.getData(file);
				var safeView = new XPCNativeWrapper(defaultView);
				var sandbox = Components.utils.Sandbox(safeView);
				sandbox.window = safeView;
				sandbox.document = safeView.document;
				sandbox.XPathResult = Components.interfaces.nsIDOMXPathResult;
				sandbox.__proto__ = safeView;
				lineFinder = new Error();
				Components.utils.evalInSandbox(data, sandbox);
			}catch(e){
				if (!lineFinder) throw e;
				else
				{
				        var consoleService = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
					var scriptError = Components.classes['@mozilla.org/scripterror;1'].createInstance(Components.interfaces.nsIScriptError);
					var sourceName = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService).newFileURI(file).spec;
					scriptError.init(e.message, sourceName, null, e.lineNumber - lineFinder.lineNumber, e.columnNumber, scriptError.errorFlag, "content javascript");
					consoleService.logMessage(scriptError);
				}
			}
		}
	};

	this.onclick = function(event, sw, st, isBooleanSwitch)
	{
		if (event && event.button && event.button > 0) return;
		var switchID = sw.getID();
		if (isBooleanSwitch)
		{
			st = sw.getState(0);
			var stateID = st.getID();
			var value = this.control.getValue(ROOT + switchID + "." + VALUE, stateID, STRING);
			if (value == stateID) st = sw.getState(1);
		}
		for (var i = 0; i < st.getPreferences().length; i++)
		{
			var pr = st.getPreference(i);
			if (pr.isHidden()) continue;
			this.control.setValue(pr.getName(), pr.getValue(), pr.getType());
		}
		this.control.setValue(ROOT + switchID + "." + VALUE, st.getID(), STRING);
	};

	this.updateProfiles = function()
	{
		while (this.profileMenupopup.firstChild.getAttribute("id") != ID_PROFILE_SEPARATOR) this.profileMenupopup.removeChild(this.profileMenupopup.firstChild);
		var profileID = this.control.getValue(PROFILE_ID, null, STRING);
		var currentProfile = null;
		var aProfile = this.control.getProfiles();
		while (aProfile.length > 0)
		{
			var profile = aProfile.shift();
			var menuitem = document.createElement("menuitem");
			menuitem.setAttribute("type", "radio");
			menuitem.setAttribute("name", "prefswitch-profile-menuitem");
			menuitem.setAttribute("label", profile.getName());
			menuitem.profile = profile;
			menuitem.setAttribute("oncommand", "prefswitchOverlay.loadProfile(this.profile)");
			if (profileID == profile.getID())
			{
				menuitem.setAttribute("checked", "true");
				currentProfile = profile;
			}
			this.profileMenupopup.insertBefore(menuitem, this.profileSeparator);
		}
		return currentProfile;
	};

	this.loadProfile = function(profile)
	{
		if (!profile) return;
		var statusbarpanel = this.separator.previousSibling;
		while (statusbarpanel.getAttribute("class") == CLASS_INITIAL)
		{
			var sw = statusbarpanel.sw;
			var feature = profile.getFeature(sw.getID());
			if (feature)
			{
				var st = sw.getStateByID(feature.value);
				statusbarpanel.setAttribute("hidden", "" + !feature.show);
				statusbarpanel.mi.setAttribute("checked", "" + feature.show);
				if (!feature.disabled) this.onclick(LOAD_PROFILE, sw, st, false);
			}
			statusbarpanel = statusbarpanel.previousSibling;
		}
		this.control.setValue(PROFILE_ID, profile.getID(), STRING);
	};
}

function $(id) {return document.getElementById(id);}

var prefswitchOverlay = new Overlay();
window.addEventListener("load", prefswitchOverlay, false);
/*window.addEventListener("unload", prefswitchOverlay, false);*/
