
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function ProfileDialog()
{
	this.control = null;
	this.listbox = null;
	this.dialogheader = null;
	this.vbox = null;
	this.listitem = null;
	this.profile = null;

	this.click = function(switchID, stateID, show, disabled)
	{
		if (!this.profile) return;
		this.profile.setFeature(switchID, stateID, show, disabled);
	};

	this.disableFeature = function(switchID)
	{
		if (!this.profile) return;
		var feature = this.profile.getFeature(switchID);
		feature.disabled = !($("disabled-" + switchID).getAttribute("checked") == "true");
//		$("show-" + switchID).setAttribute("disabled", feature.disabled);
		$(switchID).setAttribute("disabled", feature.disabled);
	};

	this.newProfile = function()
	{
		this.profile = new Profile(null, "New Profile");
		var listitem = this.listbox.appendItem(this.profile.getName());
		listitem.profile = this.profile;
		this.listbox.selectedIndex = this.listbox.getRowCount() - 1;
		this.refreshProfiles();
		if (!this.renameProfile())
		{
			this.listbox.removeItemAt(this.listbox.selectedIndex);
			if (this.listbox.getRowCount() > 0) this.listbox.selectedIndex = 0;
			else this.refreshProfiles();
		}
	};

	this.renameProfile = function()
	{
		if (this.profile == null) return false;
		var name = prompt("Enter name of profile:", this.profile.getName());
		if (name == null || name == "") return false;
		this.profile.setName(name);
		this.refreshProfiles();
		return true;
	};

	this.removeProfile = function()
	{
		if (this.listbox.selectedIndex < 0) return;
		this.listbox.removeItemAt(this.listbox.selectedIndex);
		if (this.listbox.getRowCount() > 0) this.listbox.selectedIndex = 0;
	};

	this.load = function()
	{
		if ("arguments" in window && window.arguments.length > 0) this.control = window.arguments[0];
		else this.control = new Control();
		this.listbox = $("listbox");
		this.dialogheader = $("dialogheader");
		this.vbox = $("vbox");
		this.listbox.addEventListener("select", this, false);
		this.loadMenulists();
		this.loadProfiles();
		if (this.listbox.getRowCount() > 0) this.listbox.selectedIndex = 0;
	};

	this.handleEvent = function(event)
	{
		this.refreshProfiles();
	};

	this.loadMenulists = function()
	{
		var aSwitch = this.control.getSwitches();
		var mySwitch = aSwitch.shift();
		while (mySwitch)
		{
			if (mySwitch.isHidden())
			{
				mySwitch = aSwitch.shift();
				continue;
			}
			var switchID = mySwitch.getID();
			var hbox = document.createElement("hbox");
			hbox.setAttribute("align", "center");

			// CREATE SHOW/HIDE MENULIST
			var menulist = document.createElement("menulist");
			menulist.setAttribute("id", "show-" + switchID);
			menulist.setAttribute("name", switchID);
			var menupopup = document.createElement("menupopup");
			var menuitem = document.createElement("menuitem");

			var strbundle = document.getElementById("stringbundle");

			menuitem.setAttribute("label", strbundle.getString("menuitemShow"));
			menuitem.setAttribute("value", "true");
			menuitem.setAttribute(
				"oncommand",
				"profileDialog.click(\"" + switchID + "\", $(\"" + switchID + "\").selectedItem.getAttribute('id'), true, false)");
			menupopup.appendChild(menuitem);
			menuitem = document.createElement("menuitem");
			menuitem.setAttribute("label", strbundle.getString("menuitemHide"));
			menuitem.setAttribute("value", "false");
			menuitem.setAttribute(
				"oncommand",
				"profileDialog.click(\"" + switchID + "\", $(\"" + switchID + "\").selectedItem.getAttribute('id'), false, false)");
			menupopup.appendChild(menuitem);
			menulist.appendChild(menupopup);
			hbox.appendChild(menulist);

			// CREATE DISABLE/ENABLE CHECKBOX
			var checkbox = document.createElement("checkbox");
			checkbox.setAttribute("id", "disabled-" + switchID);
			checkbox.setAttribute("label", "");
//			checkbox.setAttribute("checked", "false");
			checkbox.setAttribute("oncommand", "profileDialog.disableFeature(\"" + switchID + "\")");
			hbox.appendChild(checkbox);

			// CREATE SWITCH MENULIST
			var label = document.createElement("label");
			label.setAttribute("value", mySwitch.getName() + ":");
			hbox.appendChild(label);
			menulist = document.createElement("menulist");
			menulist.setAttribute("id", switchID);
			menulist.setAttribute("name", switchID);
			menulist.setAttribute("label", mySwitch.getName());
			var menupopup = document.createElement("menupopup");
			var aState = mySwitch.getStates();
			var tState = aState.shift();
			while (tState)
			{
				var menuitem = document.createElement("menuitem");
				menuitem.setAttribute("id", tState.getID());
				menuitem.setAttribute("label", tState.getName());
				menuitem.setAttribute(
					"oncommand",
					"profileDialog.click(\"" + switchID + "\", \"" + tState.getID() + "\", $(\"show-" + switchID + "\").selectedItem.getAttribute('value'))");
				menupopup.appendChild(menuitem);
				tState = aState.shift();
			}
			menulist.appendChild(menupopup);
			hbox.appendChild(menulist);
			this.vbox.appendChild(hbox);
			mySwitch = aSwitch.shift();
		}
	};

	this.loadProfiles = function()
	{
		var aProfile = this.control.getProfiles();
		for (var i = 0; i < aProfile.length; i++)
		{
			var pr = aProfile[i];
			var listitem = this.listbox.appendItem(pr.getName());
			listitem.profile = pr;
		}
		if (this.listbox.getRowCount() > 0 && this.listbox.selectedIndex < 0) this.listbox.selectedIndex = 0;
	};

	this.refreshProfiles = function()
	{
		if (this.listbox.selectedIndex < 0)
		{
			this.listitem = null;
			this.profile = null;
			this.dialogheader.setAttribute("title", "");
		}
		else
		{
			this.listitem = this.listbox.getSelectedItem(0);
			if (!this.listitem) return;
			this.profile = this.listitem.profile;
			if (!this.profile) return;
			this.listitem.setAttribute("label", this.profile.getName());
			this.dialogheader.setAttribute("title", this.profile.getName());

			// SELECT MENUITEMS
			var aMenulist = document.getElementsByTagName("menulist");
			for (var i = 0; i < aMenulist.length; i++)
			{
				var menulist = aMenulist[i];
				if (menulist.getAttribute("id").indexOf("show-") == 0) continue;
				var switchID = menulist.getAttribute("name");
				var feature = this.profile.getFeature(switchID);
				if (feature)
				{
					if (feature.disabled)
					{
//						$("show-" + switchID).setAttribute("disabled", "true");
						$("disabled-" + switchID).setAttribute("checked", "false");
						$(switchID).setAttribute("disabled", "true");
					}
					else
					{
//						$("show-" + switchID).setAttribute("disabled", "false");
						$("disabled-" + switchID).setAttribute("checked", "true");
						$(switchID).setAttribute("disabled", "false");
					}

					if (feature.show) $("show-" + switchID).selectedIndex = 0;
					else $("show-" + switchID).selectedIndex = 1;

					var menupopup = $(switchID).firstChild;
					var menuitem = menupopup.firstChild;
					while (menuitem)
					{
						if (feature.value == menuitem.getAttribute("id"))
						{
							$(switchID).selectedItem = menuitem;
							break;
						}
						menuitem = menuitem.nextSibling;
					}
				}
				else
				{
					menulist.selectedIndex = 0;
					this.profile.setFeature(switchID, $(switchID).selectedItem.getAttribute("id"), true);
				}
			}
		}
	};

	this.accept = function()
	{
		var data = this.generateXml();
		this.control.saveProfiles(data);
		this.control.update(UPDATE_PROFILES);
	};

	this.cancel = function()
	{
	};

	this.generateXml = function()
	{
		var r =
"<?xml version=\"1.0\" encoding=\"" + CHARSET + "\"?>\n" +
"<prefswitch>\n";
		for (var i = 0; i < this.listbox.getRowCount(); i++)
		{
			var pr = this.listbox.getItemAtIndex(i).profile;
			if (!pr) continue;
			r +=
"	<profile" +
		" id=\"" + pr.getID() + "\"" +
		" name=\"" + this.control.encode(pr.getName()) + "\"" +
		">\n";
			for (var key in pr.getFeatures())
			{
				var feature = pr.getFeature(key);
				r +=
"		<feature" +
			" key=\"" + key + "\"" +
			" value=\"" + feature.value + "\"" +
			" show=\"" + feature.show + "\"" +
			" disabled=\"" + feature.disabled + "\"" +
			"/>\n";
			}
			r +=
"	</profile>\n";
		}
		r +=
"</prefswitch>";
		return r;
	};
}

function $(id) {return document.getElementById(id);}

var profileDialog = new ProfileDialog();
